﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace Builder
{
    class Program
    {
        static void Main(string[] args)
        {
            LeftBuilder lb = new LeftBuilder();
            RightBuilder rb = new RightBuilder();
            Director.Build(lb);
            lb.GetProduct().ShowProduct();
            Director.Build(rb);
            rb.GetProduct().ShowProduct();

            Console.ReadLine();
        }


    }

    class Product
    {
        IList<String> parts = new List<String>();

        public void AddPart(string part)
        {
            parts.Add(part);
        }

        public void ShowProduct()
        {
            Console.WriteLine();
            foreach (string str in parts)
            {
                Console.Write(str);
            }
            Console.WriteLine();
        }
    }

    abstract class Builder
    {
        protected Product p = new Product();
        public Product GetProduct()
        {
            return p;
        }

        public abstract void BuildA();
        public abstract void BuildB();
    }

    class LeftBuilder : Builder
    {
        public override void BuildA()
        {
            p.AddPart("轮子");
        }

        public override void BuildB()
        {
            p.AddPart("架子");
        }
    }

    class RightBuilder : Builder
    {
        public override void BuildA()
        {
            p.AddPart("脚");
        }

        public override void BuildB()
        {
            p.AddPart("身体");
        }
    }

    class Director
    { 
        public static void Build(Builder b)
        {
            b.BuildA();
            b.BuildB();
        }
    }
}
