﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Singleton
{
    class Program
    {
        static void Main(string[] args)
        {
            Structure obj1 = Structure.GetInstance();
            Structure obj2 = Structure.GetInstance();
            if (obj1 == obj2)
            {
                Console.WriteLine("obj1 is the same with obj2");
            }
            else
            {
                Console.WriteLine("obj1 is diffient with obj2");
            }
            Console.WriteLine(obj1.GetLine());

            Structure2 obj3 = Structure2.GetInstance();
            Structure2 obj4 = Structure2.GetInstance();
            if (obj3 == obj4)
            {
                Console.WriteLine("obj3 is the same with obj4");
            }
            else
            {
                Console.WriteLine("obj3 is diffient with obj4");
            }
            Console.WriteLine(obj3.GetLine());

            Console.ReadLine();
        }
    }


    /// <summary>
    /// 懒汉形
    /// </summary>
    class Structure
    {
        private static Structure instance = null;
        private readonly static object lockobj = new object();

        private Structure()//防止外界实例化
        { }

        public string GetLine()
        {
            if (instance != null)
            {
                return instance.GetType().ToString();
            }
            else
            {
                return "the obj is not instance yet!";
            }
        }

        public static Structure GetInstance()
        {
            if (instance == null)
            {
                lock (lockobj)
                {
                    if (instance == null)//不可上，防止多线程进入，又重锁定
                    {
                        instance = new Structure();
                    }
                }
            }
            return instance;
        }
    }

    /// <summary>
    /// 饿汉形
    /// </summary>
    sealed class Structure2//防止派生添加实例
    {
        private static readonly Structure2 instance = new Structure2();//readonly

        private Structure2()//防止外界实例化
        { }

        public string GetLine()
        {
            return instance.GetType().ToString();
        }

        public static Structure2 GetInstance()
        {
            return instance;
        }
    }


}
